using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Data.SqlClient;
using System.Drawing;
using System.Web;
using System.Web.SessionState;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using Microsoft.Web.UI.WebControls;
using gov.va.med.vistaweb.util;

namespace UserSites
{
	/// <summary>
	/// Summary description for Sites.
	/// </summary>
	public class Sites : System.Web.UI.Page
	{
		protected System.Web.UI.WebControls.DataGrid DataGrid1;
		protected System.Web.UI.WebControls.DropDownList ReasonList;
		protected System.Web.UI.WebControls.TextBox OtherReason;
		protected System.Web.UI.WebControls.Button Submit;
		protected Microsoft.Web.UI.WebControls.TreeView SiteTree;
		protected System.Web.UI.WebControls.CheckBox chkAllSites;
		protected System.Web.UI.WebControls.Button Reset;
		protected System.Web.UI.WebControls.TextBox StatusBox;
	
		private void Page_Load(object sender, System.EventArgs e)
		{
			Response.CacheControl = "no-cache";
			Response.AddHeader("Pragma", "no-cache");
			Response.Expires = -1;

			if (!Page.IsPostBack) 
			{
				MDO.SiteTable siteTable = (MDO.SiteTable)Session["VhaSiteTable"];
				populateTree(SiteTree,siteTable,"Sites.aspx");
				defineGrid();
				initGrid();
			}
			handleTreeSelection();
		}

		public static void populateTree(TreeView SiteTree, MDO.SiteTable siteTable, String URL) 
		{
			TreeNode root = new TreeNode();
			root.Text = "VHA";
			root.NavigateUrl = "";
			root.Expanded = true;
			SiteTree.Nodes.Add(root);

			MDO.Region[] regions = siteTable.getRegions();
			String defaultCss = "color:black;font-weight:bold;font-family:georgia;font-size:8pt";
			String selectedCss = "color:red;font-weight:bold;font-family:georgia;font-size:8pt";
			String hoverCss = "color:yellow;font-weight:bold;font-family:georgia;font-size:8pt;text-decoration:none";
			for (int r=0; r<regions.Length; r++) 
			{
				TreeNode regionNode = new TreeNode();
				regionNode.Text = regions[r].getID() + ": " + regions[r].getName();
				regionNode.DefaultStyle.CssText = defaultCss;
				regionNode.SelectedStyle.CssText = selectedCss;
				regionNode.HoverStyle.CssText = hoverCss;
				regionNode.NavigateUrl = "";
				MDO.Site[] sites = regions[r].getSites();
				for (int s=0; s<sites.Length; s++) 
				{
					TreeNode siteNode = new TreeNode();
					siteNode.Text = sites[s].getDisplayName();
					siteNode.DefaultStyle.CssText = defaultCss;
					siteNode.SelectedStyle.CssText = selectedCss;
					siteNode.HoverStyle.CssText = hoverCss;
					siteNode.ID = sites[s].getID();
					siteNode.NavigateUrl = "";
					siteNode.Target = "WorkPanel";
					regionNode.Nodes.Add(siteNode);
				}
				root.Nodes.Add(regionNode);
			}
		}

		private void initGrid() 
		{
			DataTable t = (DataTable)Session["UserSiteTable"];
			MDO.DataSourceDAO loginDao = (MDO.DataSourceDAO)Session["LoginDao"];
			String siteID = loginDao.getSite().getID();
			MDO.User provider = (MDO.User)Session["Provider"];
			SqlConnection conn = (SqlConnection)Session["Connection"];
			String sql = "SELECT * FROM SpecialUsers " +
				"WHERE DUZ='" + provider.getID() + "' AND UserSiteID='" + provider.getSite().getID() + "';";
			SqlCommand cmd = new SqlCommand(sql,conn);
			SqlDataReader rdr = null;
			bool fAllSites = false;
			try 
			{
				conn.Open();
				rdr = cmd.ExecuteReader();
				int idx = rdr.GetOrdinal("DeactiveDate");
				while (rdr.Read()) 
				{
					DataRow dataRow = t.NewRow();
					dataRow["RecID"] = rdr["RecID"];
					string sSite = (string)rdr["Site"];
					dataRow["Site"] = sSite;
					dataRow["Reason"] = rdr["Reason"];
					dataRow["ActiveDate"] = rdr["ActiveDate"];
					string sDeactiveDate = (rdr.IsDBNull(idx) ? "" : rdr.GetDateTime(idx).ToString());
					dataRow["DeactiveDate"] = sDeactiveDate;
					dataRow["Op"] = "N";
					t.Rows.Add(dataRow);
					if (sSite == "VHA" && sDeactiveDate == "") fAllSites = true;
				}
				rdr.Close();
			}
			catch (Exception ex) 
			{
				Response.Write("Error getting lookup site codes: " + ex.Message);
				Response.End();
			}
			finally
			{
				conn.Close();
			}
			conn = null;
			rdr = null;
			bindGrid(t);
			Session["UserSiteTable"] = t;
			if (fAllSites) 
			{
				chkAllSites.Checked = true;
				SiteTree.Visible = false;
			}
		}

		private void defineGrid() 
		{
			DataTable t = new DataTable();
			t.Columns.Add(new DataColumn("RecID",Type.GetType("System.String")));
			t.Columns.Add(new DataColumn("Site",Type.GetType("System.String")));
			t.Columns.Add(new DataColumn("Reason",Type.GetType("System.String")));
			t.Columns.Add(new DataColumn("ActiveDate",Type.GetType("System.String")));
			t.Columns.Add(new DataColumn("DeactiveDate",Type.GetType("System.String")));
			t.Columns.Add(new DataColumn("Op",Type.GetType("System.String")));
			Session["UserSiteTable"] = t;
		}

		private void handleTreeSelection() 
		{
			String idx = SiteTree.SelectedNodeIndex;
			if (idx == "") return;
			String[] flds = idx.Split('.');
			if (flds.Length == 1) return;
			int visnIdx = Convert.ToInt32(flds[1]);
			MDO.SiteTable siteTable = (MDO.SiteTable)Session["VhaSiteTable"];
			MDO.Region visn = siteTable.getRegions()[visnIdx];
			MDO.Site site = null;
			if (flds.Length == 3) 
			{
				int siteIdx = Convert.ToInt32(flds[2]);
				site = visn.getSites()[siteIdx];
				addSite(site);
			}
			else 
			{
				addVisn(visn);
			}
			SiteTree.SelectedNodeIndex = "";
		}

		private void addSite(MDO.Site site) 
		{
			if (alreadyHaveVisn(site)) 
			{
				StatusBox.Text = "Access to this site already granted in VISN " + site.getRegionID() + ".";
				return;
			}
			DataTable t = (DataTable)Session["UserSiteTable"];
			string siteStr = "SITE " + site.getID();
			addRec(t,siteStr);
		}

		private void addVisn(MDO.Region visn) 
		{
			deactivateSites(visn);
			DataTable t = (DataTable)Session["UserSiteTable"];
			string siteStr = "VISN " + visn.getID();
			addRec(t,siteStr);
		}

		private void addRec(DataTable t, string siteStr) 
		{
			if (alreadyHaveRecord(t,siteStr)) return;
			DataRow dataRow = t.NewRow();
			dataRow["RecID"] = "";
			dataRow["Site"] = siteStr;
			dataRow["Reason"] = "";
			dataRow["ActiveDate"] = DateTime.Now.ToString();
			dataRow["DeactiveDate"] = "";
			dataRow["Op"] = "I";
			t.Rows.Add(dataRow);
			bindGrid(t);
		}

		private String getReason() 
		{
			String reason = ReasonList.SelectedItem.Value;
			if (reason == "Other")
				reason = OtherReason.Text;
			return reason;
		}

		#region Web Form Designer generated code
		override protected void OnInit(EventArgs e)
		{
			//
			// CODEGEN: This call is required by the ASP.NET Web Form Designer.
			//
			InitializeComponent();
			base.OnInit(e);
		}
		
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{    
			this.chkAllSites.CheckedChanged += new System.EventHandler(this.chkAllSites_CheckedChanged);
			this.DataGrid1.EditCommand += new System.Web.UI.WebControls.DataGridCommandEventHandler(this.DataGrid1_EditCommand);
			this.DataGrid1.DeleteCommand += new System.Web.UI.WebControls.DataGridCommandEventHandler(this.DataGrid1_DeleteCommand);
			this.Submit.Click += new System.EventHandler(this.Submit_Click);
			this.Reset.Click += new System.EventHandler(this.Reset_Click);
			this.Load += new System.EventHandler(this.Page_Load);

		}
		#endregion

		private void DataGrid1_DeleteCommand(object source, System.Web.UI.WebControls.DataGridCommandEventArgs e)
		{
			DataTable t = (DataTable)Session["UserSiteTable"];
			string sDate = (string)t.Rows[e.Item.ItemIndex]["DeactiveDate"];
			if (sDate != "") return;
			if ((string)t.Rows[e.Item.ItemIndex]["Op"] == "I")
				t.Rows[e.Item.ItemIndex].Delete();
			else
				deactivateRecord(t,e.Item.ItemIndex,DateTime.Now.ToString());
			bindGrid(t);
		}

		private void DataGrid1_EditCommand(object source, System.Web.UI.WebControls.DataGridCommandEventArgs e)
		{
			DataTable t = (DataTable)Session["UserSiteTable"];
			t.Rows[e.Item.ItemIndex]["Reason"] = getReason();
			if ((string)t.Rows[e.Item.ItemIndex]["Op"] == "N")
				t.Rows[e.Item.ItemIndex]["Op"] = "U";
			bindGrid(t);
		}

		private void Submit_Click(object sender, System.EventArgs e)
		{
			SqlConnection conn = (SqlConnection)Session["Connection"];
			SqlConnection connCard = (SqlConnection)Session["ConnCard"];
			MDO.User provider = (MDO.User)Session["Provider"];
			MDO.DataSourceDAO lookupDao = (MDO.DataSourceDAO)Session["LookupDao"];
			if (blankReasons()) 
			{
				StatusBox.Text = "Cannot save with blank reasons.";
				return;
			}

			conn.Open();
			connCard.Open();
			DataTable t = (DataTable)Session["UserSiteTable"];
			for (int i=0; i<t.Rows.Count; i++) 
			{
				string op = (string)t.Rows[i]["OP"];
				if (op == "N") continue;
				string sql = "";
				string sUser = "";
				if (!provider.getLastName().Equals(""))
				{
					sUser = provider.getName();
				}
				else
				{
					sUser = provider.getVistaFullName();
				}
				if (op == "I") 
				{
					sql = "INSERT INTO SpecialUsers (UserSiteID, DUZ, UserName, Site, Reason, ActiveDate) " +
						"VALUES ('" + lookupDao.getSite().getID() + "','" + provider.getID() + "','" +
						sUser + "', '" + (string)t.Rows[i]["Site"] + "','" + (string)t.Rows[i]["Reason"] + "','" +
						(string)t.Rows[i]["ActiveDate"] + "');";
				} 
				else if (op == "U") 
				{
					sql = "UPDATE SpecialUsers SET Reason='" + (string)t.Rows[i]["Reason"] + 
						"', UserName = '" + sUser + "' " +
						"WHERE RecID=" + t.Rows[i]["RecID"] + ";";
				}
				else 
				{
					sql = "UPDATE SpecialUsers SET DeactiveDate='" + (string)t.Rows[i]["DeactiveDate"] + 
						"', UserName = '" + sUser + "' " +
						"WHERE RecID=" + t.Rows[i]["RecID"] + ";";
				}
				SqlCommand cmd = new SqlCommand(sql,conn);
				SqlCommand cmdCard = new SqlCommand(sql, connCard);
				try 
				{
					cmd.ExecuteNonQuery();
					cmdCard.ExecuteNonQuery();
				} 
				catch (Exception ex) 
				{
					Response.Write("Unable to update record for " + (string)t.Rows[i]["Site"] + " - aborting: " + ex.Message);
					Response.End();
				}
			}
			setNoOps();
			StatusBox.Text = "Record(s) saved.";
			conn.Close();
			connCard.Close();
		}

		private bool blankReasons() 
		{
			DataTable t = (DataTable)Session["UserSiteTable"];
			for (int i=0; i<t.Rows.Count; i++) 
			{
				if ((string)t.Rows[i]["Reason"] == "") return true;
			}
			return false;
		}

		private void deactivateRecord(int idx) 
		{
			DataTable t = (DataTable)Session["UserSiteTable"];
			string sDate = DateTime.Now.ToString();
			deactivateRecord(t,idx,sDate);
		}

		private void deactivateRecord(DataTable t, int idx, string sDate) 
		{
			t.Rows[idx]["DeactiveDate"] = sDate;
			t.Rows[idx]["Op"] = "D";
		}

		private void chkAllSites_CheckedChanged(object sender, System.EventArgs e)
		{
			DataTable t = (DataTable)Session["UserSiteTable"];
			if (chkAllSites.Checked) 
			{
				if (alreadyHaveRecord(t,"VHA")) return;
				SiteTree.Visible = false;
				string sDate = DateTime.Now.ToString();
				for (int i=0; i<t.Rows.Count; i++) 
				{
					deactivateRecord(t,i,sDate);
				}
				DataRow dataRow = t.NewRow();
				dataRow["Site"] = "VHA";
				dataRow["Reason"] = "";
				dataRow["ActiveDate"] = sDate;
				dataRow["DeactiveDate"] = "";
				dataRow["Op"] = "I";
				t.Rows.Add(dataRow);
				bindGrid(t);
			} 
			else 
			{
				int idx = t.Rows.Count - 1;
				if ((string)t.Rows[idx]["Site"] != "VHA") 
				{
					Response.Write("Last record should be VHA and isn't");
					Response.End();
				}
				t.Rows[idx]["DeactiveDate"] = DateTime.Now.ToString();
				t.Rows[idx]["Op"] = "D";
				bindGrid(t);
				SiteTree.Visible = true;
			}
		}

		private void bindGrid(DataTable t) 
		{
			DataView dataView = new DataView(t);
			DataGrid1.DataSource = dataView;
			DataGrid1.DataBind();
			StatusBox.Text = "";
		}

		private bool alreadyHaveRecord(DataTable t, string site) 
		{
			for (int i=0; i<t.Rows.Count; i++) 
			{
				if ((string)t.Rows[i]["Site"] != site) continue;
				if ((string)t.Rows[i]["DeactiveDate"] == "") return true;
			}
			return false;
		}

		private void setNoOps() 
		{
			DataTable t = (DataTable)Session["UserSiteTable"];
			for (int i=0; i<t.Rows.Count; i++) 
			{
				t.Rows[i]["Op"] = "N";
			}
			bindGrid(t);
		}

		private void Reset_Click(object sender, System.EventArgs e)
		{
			DataTable t = (DataTable)Session["UserSiteTable"];
			t.Clear();
			initGrid();
		}

		private void deactivateSites(MDO.Region visn) 
		{
			DataTable t = (DataTable)Session["UserSiteTable"];
			MDO.Site[] sites = visn.getSites();
			for (int i=0; i<sites.Length; i++) 
			{
				int j = siteIdx(sites[i].getID());
				if (j == -1) continue;
				string sOp = (string)t.Rows[j]["Op"];
				string sDate = (string)t.Rows[j]["DeactiveDate"];
				if (sDate != "") continue;
				if (sOp == "I") 
				{
					t.Rows[j].Delete();
					continue;
				}
				deactivateRecord(j);
			}
			bindGrid(t);
		}

		private int siteIdx(string siteCode) 
		{
			string siteID = "SITE " + siteCode;
			DataTable t = (DataTable)Session["UserSiteTable"];
			for (int i=0; i<t.Rows.Count; i++) 
			{
				string val = (string)t.Rows[i]["Site"];
				if (val == siteID) return i;
			}
			return -1;
		}

		private bool alreadyHaveVisn(MDO.Site site) 
		{
			string visnID = "VISN " + site.getRegionID();
			DataTable t = (DataTable)Session["UserSiteTable"];
			for (int i=0; i<t.Rows.Count; i++) 
			{
				string val = (string)t.Rows[i]["Site"];
				string sDate = (string)t.Rows[i]["DeactiveDate"];
				if (val == visnID && sDate == "") return true;
			}
			return false;
		}
	}
}
